/*
    *
    * This file is a part of CorePins.
    * A bookmarking app for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/


#include <QFile>
#include <QFileDialog>
#include <QMimeData>
#include <QMouseEvent>
#include <QMessageBox>

#include <cprime/sortfunc.h>
#include <cprime/infofunc.h>
#include <cprime/themefunc.h>
#include <cprime/settingsmanage.h>
#include <cprime/pinit.h>
#include <cprime/appopenfunc.h>

#include "corepins.h"
#include "ui_corepins.h"


corepins::corepins(QWidget *parent): QWidget(parent), ui(new Ui::corepins)
{
    ui->setupUi(this);

    QPalette pltt(palette());
    pltt.setColor(QPalette::Base, Qt::transparent);
    setPalette(pltt);

    loadSettings();
    startSetup();

    pm.checkPins();
    sectionRefresh();
    ui->pinlist->viewport()->installEventFilter(this);
}

corepins::~corepins()
{
    delete ui;
}

/**
 * @brief Setup ui elements
 */
void corepins::startSetup()
{
    // set stylesheet
    ui->sideView->setStyleSheet(CPrime::ThemeFunc::getSideViewStyleSheet());
    ui->sideView->setFixedWidth(sideViewSize);

    // set window size
    if (touch) {
        this->setWindowState(Qt::WindowMaximized);
    } else {
        this->resize(800, 500);
    }

    // all toolbuttons icon size in sideView
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        if (b) {
            b->setIconSize(sideViewIconSize);
        }
    }

    // all toolbuttons icon size in topBar
    for (QToolButton *b : ui->topBar->findChildren<QToolButton *>()) {
        if (b) {
            b->setIconSize(toolBarIconSize);
        }
    }

    on_appTitle_clicked();

    ui->pinlist->setIconSize(listViewIconSize);
    ui->pinlist->verticalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
    ui->section->setIconSize(sideViewIconSize);

    if (touch) {
        connect(ui->pinlist, SIGNAL(itemClicked(QTableWidgetItem *)), this, SLOT(openSelectedBookmark(QTableWidgetItem *)));
    } else {
        connect(ui->pinlist, SIGNAL(itemDoubleClicked(QTableWidgetItem *)), this, SLOT(openSelectedBookmark(QTableWidgetItem *)));
    }

    bool count = false;
    int c = pm.getPinSections().count();

    // When no section added but speed dial contains item.
    if (c == 1 && pm.getPinNames("Speed Dial").count() >= 1) {
        count = true;
    }

    // When speed dial contains no item but other section do.
    if (c > 1) {
        foreach (QString sec, pm.getPinSections()) {
            if (pm.getPinNames(sec).count()) {
                count = true;
                break;
            }
        }
    }

    if (count) {
        ui->page->setCurrentIndex(1);
    } else {
        ui->page->setCurrentIndex(0);
    }

    ui->pinEditBox->setVisible(false);
    ui->addPin->setVisible(false);
    ui->addSectionBox->setVisible(false);
    ui->deleteSection->setVisible(false);
    ui->addSection->setVisible(false);
    ui->cTools->setChecked(false);
    ui->pinlist->setFocusPolicy(Qt::NoFocus);
}

/**
 * @brief Loads application settings
 */
void corepins::loadSettings()
{    
    // Sideview is visible in default
    sideView = 1;

    bAddPin = false;
    sideView = true;
    sideViewSize = sm->value("CoreApps", "SideViewSize");
    toolBarIconSize = sm->value("CoreApps", "ToolBarIconSize");
    listViewIconSize = sm->value("CoreApps", "ListViewIconSize");
    sideViewIconSize = sm->value("CoreApps", "SideViewIconSize");
    touch = sm->value("CoreApps", "TouchMode");
}

bool corepins::eventFilter(QObject *obj, QEvent *event)
{
    if (obj == ui->pinlist->viewport()) {
        if (event->type() == QEvent::MouseButtonDblClick) {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);

            if (mouseEvent->button() == Qt::RightButton) {
                return true;
            } else {
                return false;
            }
        }
    }

    return QWidget::eventFilter(obj, event);
}

void corepins::changeState(bool state)
{
    ui->section->setEnabled(state);
    ui->cTools->setChecked(false);
    ui->cTools->setEnabled(state);
    ui->reload->setEnabled(state);
    ui->searchPins->setEnabled(state);
    ui->pinlist->setEnabled(state);
}

void corepins::sectionRefresh()
{
    disconnect(ui->searchPins, SIGNAL(textChanged(QString)), nullptr, nullptr);

    ui->pinlist->clear();
    ui->searchPins->clear();

    int selectedIndex = ui->section->currentIndex().row();

    ui->section->clear();
    ui->section->addItems(pm.getPinSections());

    for (int i = 0; i < ui->section->count(); ++i) {
        QIcon icon = QIcon::fromTheme("address-book-new");
        ui->section->item(i)->setIcon(icon);
    }

    ui->section->setCurrentRow(selectedIndex);

    if (selectedIndex > -1) {
        on_section_itemClicked(ui->section->currentItem());
    } else if (selectedIndex == -1) {
        ui->section->setCurrentRow(0);
        on_section_itemClicked(ui->section->currentItem());
    }

    connect(ui->searchPins, SIGNAL(textChanged(QString)), this, SLOT(on_searchPins_textChanged(QString)));
}

void corepins::on_reload_clicked()
{
    sectionRefresh();
}

void corepins::on_selectSection_currentIndexChanged(const QString &arg1)
{
    Q_UNUSED(arg1)
    on_pinPath_textChanged(ui->pinPath->text());
    on_pinName_textChanged(ui->pinName->text());
}

void corepins::on_section_itemClicked(QListWidgetItem *item)
{
    if (ui->section->currentItem()->text() == "Speed Dial") {
        ui->deleteSection->setEnabled(false);
    } else {
        ui->deleteSection->setEnabled(true);
    }

    this->setWindowTitle(item->text() + " - CorePins");

    QStringList list = pm.getPinNames(item->text());
    int count = list.count();

    ui->pinlist->clear();

    ui->pinlist->setColumnCount(3);
    ui->pinlist->setRowCount(count);

    QStringList dateTimeList;

    for (int i = 0; i < count; i++) {
        dateTimeList.append(pm.piningTime(ui->section->currentItem()->text(), list.at(i)));
    }

    CPrime::SortFunc::sortDate(dateTimeList);
    QStringList mList;
    mList.clear();
    int reverse = count - 1;

    for (int i = 0; i < count; i++) {
        for (int j = 0; j < count; j++) {
            QString bTime = pm.piningTime(ui->section->currentItem()->text(), list.at(j));

            if (bTime.contains(dateTimeList.at(i))) {
                mList.insert(reverse, list.at(j));
                reverse--;
            }
        }
    }

    dateTimeList.clear();
    list.clear();

    QTableWidgetItem *pinItem;

    for (int i = 0; i < count; i++) {
        pinItem = new QTableWidgetItem(mList.at(i));
        QIcon icon = CPrime::ThemeFunc::getFileIcon(pm.pinPath(ui->section->currentItem()->text(), mList.at(i)));
        QSize cSize = ui->pinlist->iconSize();          // Correct size
        QSize iSize = icon.actualSize(cSize);           // iconSize

        if (iSize.width() > cSize.width() or iSize.height() > cSize.height()) {
            icon = QIcon(icon.pixmap(cSize).scaled(cSize, Qt::KeepAspectRatio, Qt::SmoothTransformation));
        }

        pinItem->setIcon(icon);

        ui->pinlist->setItem(i, 0, pinItem);
        ui->pinlist->setItem(i, 1, new QTableWidgetItem(pm.pinPath(ui->section->currentItem()->text(), mList.at(i))));

        pinCellItemWidget *itemWid = new pinCellItemWidget(pinItem, this);
        ui->pinlist->setCellWidget(i, 2, itemWid);

        connect(itemWid, &pinCellItemWidget::editOccurred, this, &corepins::pinEdit);
        connect(itemWid, &pinCellItemWidget::removeOccurred, this, &corepins::pinDelete);
    }

    int s = ui->pinlist->horizontalHeader()->size().width();
    int j = static_cast<int>(s * .1);
    ui->pinlist->horizontalHeader()->setMinimumSectionSize(j);
    ui->pinlist->horizontalHeader()->setSectionResizeMode(0, QHeaderView::Stretch);
    ui->pinlist->horizontalHeader()->setSectionResizeMode(1, QHeaderView::Stretch);
    ui->pinlist->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Fixed);
}

void corepins::on_addSection_clicked()
{
    if (ui->pinEditBox->isVisible()) {
        ui->pinEditBox->setVisible(false);
        ui->selectSection->clear();
    }

    ui->addSectionBox->setVisible(true);
    ui->sectionDone->setEnabled(false);
}

void corepins::on_deleteSection_clicked()
{
    QMessageBox message(QMessageBox::Question, tr("Delete Section"), "Do you want to delete this section?", QMessageBox::No | QMessageBox::Yes);
    message.setWindowIcon(QIcon("corepins"));

    int merge = message.exec();

    if (merge == QMessageBox::Yes) {
        pm.delSection(ui->section->currentItem()->text());
        ui->section->takeItem(ui->section->currentIndex().row());
    }

    sectionRefresh();
}

void corepins::on_sectionDone_clicked()
{
    QString sectionName = ui->sectionName->text();
    pm.addSection(sectionName);

    QListWidgetItem *item = new QListWidgetItem(sectionName);
    item->setIcon(QIcon::fromTheme("address-book-new"));
    ui->section->addItem(item);

    ui->sectionName->setText("");
    ui->sectionStatus->setText("");
    ui->addSectionBox->setVisible(false);

    on_cTools_clicked();
}

void corepins::on_sectionCancel_clicked()
{
    ui->sectionName->setText("");
    ui->sectionStatus->setText("");
    ui->addSectionBox->setVisible(false);
    on_cTools_clicked();
}

void corepins::on_sectionName_textChanged(const QString &arg1)
{
    if (arg1.count() > 0) {
        if (pm.getPinSections().contains(arg1, Qt::CaseInsensitive)) {
            ui->sectionStatus->setText("Section Exists");
            ui->sectionDone->setEnabled(false);
        } else {
            ui->sectionStatus->setText("");
            ui->sectionDone->setEnabled(true);
        }
    }

    else if (ui->sectionName->text().isEmpty()) {
        ui->sectionDone->setEnabled(false);
    }
}

void corepins::on_editDone_clicked()
{
    QString pinValue = QDir(ui->pinPath->text()).absolutePath();

    if (bAddPin) {
        pm.addPin(ui->selectSection->currentText(), ui->pinName->text(), ui->pinPath->text());
    } else {
        pm.changeAll(ui->section->currentItem()->text(), ui->pinlist->item(ui->pinlist->currentIndex().row(), 0)->text(),
                     ui->selectSection->currentText(), ui->pinName->text(), pinValue);
    }

    if (ui->section->currentItem()->text() == ui->selectSection->currentText()) {
        if (bAddPin) {
            ui->pinlist->insertRow(1);

            int index = ui->pinlist->rowCount() - 1;
            QString filePath = ui->pinPath->text();
            QIcon icon = CPrime::ThemeFunc::getFileIcon(filePath);
            QSize cSize = ui->pinlist->iconSize();
            QSize iSize = icon.actualSize(cSize);
            if (iSize.width() > cSize.width() or iSize.height() > cSize.height()) {
                icon = QIcon(icon.pixmap(cSize).scaled(cSize, Qt::KeepAspectRatio, Qt::SmoothTransformation));
            }

            QTableWidgetItem *pinItem = new QTableWidgetItem(filePath);
            pinItem->setIcon(icon);

            ui->pinlist->setItem(index, 0, pinItem);
            ui->pinlist->setItem(index, 1, new QTableWidgetItem(filePath));

            pinCellItemWidget *itemWid = new pinCellItemWidget(pinItem, this);
            ui->pinlist->setCellWidget(index, 2, itemWid);

            connect(itemWid, &pinCellItemWidget::editOccurred, this, &corepins::pinEdit);
            connect(itemWid, &pinCellItemWidget::removeOccurred, this, &corepins::pinDelete);
        } else {
            int current = ui->pinlist->currentIndex().row();
            ui->pinlist->item(current, 0)->setText(ui->pinName->text());
            ui->pinlist->item(current, 1)->setText(QDir(ui->pinPath->text()).absolutePath());
        }
    } else {
        if (bAddPin) {
            // Nothing to do
        } else {
            ui->pinlist->removeRow(ui->pinlist->currentRow());
        }
    }

    sectionRefresh();
    on_editCancel_clicked();

    changeState(true);
}

void corepins::on_editCancel_clicked()
{
    ui->pinEditBox->setVisible(false);
    ui->selectSection->clear();
    ui->pinName->clear();
    ui->statuslbl->clear();
    ui->pinPath->clear();

    changeState(true);
}

void corepins::openSelectedBookmark(QTableWidgetItem *item)
{
    QString path = pm.pinPath(ui->section->currentItem()->text(), ui->pinlist->item(item->row(), 0)->text());
    // Function from utilities.cpp
    CPrime::AppOpenFunc::appOpenEngine(path, this);
}

void corepins::on_cTools_clicked()
{
    if (ui->pinEditBox->isVisible() || ui->addSectionBox->isVisible()) {
        ui->pinEditBox->setVisible(false);
        ui->selectSection->clear();

        on_sectionCancel_clicked();
    }

    if (!ui->addSection->isVisible()) {
        ui->addPin->setVisible(true);
        ui->addSection->setVisible(true);
        ui->deleteSection->setVisible(true);
    } else {
        ui->addPin->setVisible(false);
        ui->addSection->setVisible(false);
        ui->deleteSection->setVisible(false);
    }
}

void corepins::reload()
{
    sectionRefresh();
}

void corepins::sendFiles(const QStringList &paths)
{
    foreach (QString str, paths) {
        pinit pit;
        pit.callPinIt(this, str);
    }
}

void corepins::pinEdit(QTableWidgetItem *item)
{
    ui->pinlist->setCurrentItem(item);

    if (ui->addSection->isVisible()) {
        ui->addPin->setVisible(false);
        ui->addSection->setVisible(false);
        ui->deleteSection->setVisible(false);
    }

    ui->pinEditBox->setVisible(true);
    ui->selectSection->clear();
    ui->selectSection->addItems(pm.getPinSections());
    ui->selectSection->setCurrentText(ui->section->currentItem()->text());
    ui->pinName->setText(ui->pinlist->item(ui->pinlist->currentIndex().row(), 0)->text());
    ui->pinPath->setText(ui->pinlist->item(ui->pinlist->currentIndex().row(), 1)->text());
    bAddPin = false;

    changeState(false);
}

void corepins::pinDelete(QTableWidgetItem *item)
{
    if (!item) {
        qDebug() << "Severe problem. No item found.";
        return;
    }

    ui->pinlist->setCurrentItem(item);

    QMessageBox message(QMessageBox::Question, tr("Delete Pin"), "Do you want to delete the pin?", QMessageBox::No | QMessageBox::Yes);
    message.setWindowIcon(QIcon("corepins"));

    if (ui->pinEditBox->isVisible()) {
        ui->pinEditBox->setVisible(0);
    }

    int merge = message.exec();

    if (merge == QMessageBox::Yes) {
        pm.delPin(ui->pinlist->currentItem()->text(), ui->section->currentItem()->text());
        int r = ui->pinlist->currentItem()->row();

        ui->pinlist->cellWidget(item->row(), 2)->deleteLater();
        delete item;

        ui->pinlist->removeRow(r);
    }

    sectionRefresh();
}

void corepins::on_pinName_textChanged(const QString &arg1)
{
    if (ui->pinName->text().count() > 0) {
        QString str = pm.checkingPinName(ui->selectSection->currentText(), arg1);

        if (str.count() > 0) {
            ui->statuslbl->setText(str);
            ui->editDone->setEnabled(false);
        } else {
            ui->statuslbl->setText(str);

            if (!ui->pinPath->text().isNull() && ui->pathlbl->text().count() == 0) {
                ui->editDone->setEnabled(true);
            } else {
                ui->editDone->setEnabled(false);
            }
        }
    } else {
        ui->editDone->setEnabled(false);
    }
}

void corepins::on_pinPath_textChanged(const QString &arg1)
{
    QFileInfo info(arg1);

    if (info.isDir()) {
        QDir dir(arg1);

        if (ui->pinPath->text().count() > 0) {
            if (!QDir(dir.absolutePath()).exists()) {
                ui->pathlbl->setText(("Invalid path."));
                ui->editDone->setEnabled(false);
            } else {
                QString str = pm.checkingPinPath(ui->selectSection->currentText(), dir.absolutePath());

                if (str.count() > 0) {
                    ui->pathlbl->setText(str);
                    ui->editDone->setEnabled(false);
                } else {
                    ui->pathlbl->setText(str);

                    if (ui->pinName->text().count() != 0 && ui->statuslbl->text().count() == 0) {
                        ui->editDone->setEnabled(true);
                    } else {
                        ui->editDone->setEnabled(false);
                    }
                }
            }
        } else {
            ui->editDone->setEnabled(false);
        }
    } else if (info.isFile()) {
        QFile file(arg1);

        if (ui->pinPath->text().count() > 0) {
            if (!QFile(file.fileName()).exists()) {
                ui->pathlbl->setText(("Invalid file."));
                ui->editDone->setEnabled(false);
            } else {
                QString str = pm.checkingPinPath(ui->selectSection->currentText(), file.fileName());

                if (str.count() > 0) {
                    ui->pathlbl->setText(str);
                    ui->editDone->setEnabled(false);
                } else {
                    ui->pathlbl->setText(str);

                    if (ui->pinName->text().count() != 0 && ui->statuslbl->text().count() == 0) {
                        ui->editDone->setEnabled(true);
                    } else {
                        ui->editDone->setEnabled(false);
                    }
                }
            }
        } else {
            ui->editDone->setEnabled(false);
        }
    }
}

void corepins::on_searchPins_textChanged(const QString &arg1)
{
    if (!arg1.isNull()) {
        for (int i = 0; i < ui->pinlist->rowCount(); ++i) {
            if (ui->pinlist->item(i, 0)->text().contains(arg1, Qt::CaseInsensitive)) {
                ui->pinlist->item(i, 0)->setSelected(true);
                ui->pinlist->item(i, 1)->setSelected(true);
            } else {
                ui->pinlist->item(i, 0)->setSelected(false);
                ui->pinlist->item(i, 1)->setSelected(false);
            }
        }
    }

    if (arg1.isEmpty()) {
        ui->pinlist->clearSelection();
    }
}

void corepins::on_btnBrowse_clicked()
{
    QString dir = ui->pinPath->text().count() ? QFileInfo(ui->pinPath->text()).absolutePath() : "";

    if (!dir.count()) {
        dir = QDir::homePath();
    }

    // Sideview is visible in default
    sideView = 1;
    QString filePath = QFileDialog::getOpenFileName(this, "Select File...", dir);

    if (filePath.count()) {
        ui->pinPath->setText(filePath);
    }
}

void corepins::on_addPin_clicked()
{
    QString workFilePath = QFileDialog::getOpenFileName(this, "Select File...", QDir::homePath());

    if (workFilePath.count()) {
        pinit pit;
        bool done = pit.callPinIt(this, workFilePath);
        if (done) {
            ui->page->setCurrentIndex(1);
            sectionRefresh();
        }
    }
}

void corepins::on_appTitle_clicked()
{
    if (sideView && touch) {
        // all button without text
        for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
            b->setToolButtonStyle(Qt::ToolButtonIconOnly);
            b->setFixedWidth(sideViewIconSize.width() + 10);
        }

        ui->sideView->setFixedWidth(sideViewIconSize.width() + 10);
        ui->section->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        sideView = false;
    } else {
        // all button with text
        for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
            b->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
            b->setFixedWidth(sideViewSize);
        }

        ui->sideView->setFixedWidth(sideViewSize);
        ui->section->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        sideView = true;
    }
}


void corepins::on_startAddPin_clicked()
{
    QString workFilePath = QFileDialog::getOpenFileName(this, "Select File...", QDir::homePath());

    if (workFilePath.count()) {
        pinit pit;
        bool done = pit.callPinIt(this, workFilePath);
        if (done) {
            ui->page->setCurrentIndex(1);
            sectionRefresh();
        }
    }
}

pinCellItemWidget::pinCellItemWidget(QTableWidgetItem *container, QWidget *parent)
    : QWidget(parent)
{
    m_container = container;

    QHBoxLayout *lay = new QHBoxLayout;
    QToolButton *btnEdit = new QToolButton;
    QToolButton *btnDelete = new QToolButton;

    btnEdit->setIcon(QIcon::fromTheme("document-edit"));
    btnEdit->setIconSize(sm->value("CoreApps", "ListViewIconSize") - QSize(8,8));
    btnDelete->setIcon(QIcon::fromTheme("edit-delete"));
    btnDelete->setIconSize(sm->value("CoreApps", "ListViewIconSize") - QSize(8,8));

    lay->addWidget(btnEdit);
    lay->addWidget(btnDelete);

    this->setLayout(lay);

    connect(btnEdit, &QToolButton::clicked, [this]() {
        emit editOccurred(m_container);
    });

    connect(btnDelete, &QToolButton::clicked, [this]() {
        emit removeOccurred(m_container);
    });
}
